from wagtail.api.v2.serializers import Field
from importlib import import_module
from django.conf import settings


class StreamFieldSerializer(Field):
    def __init__(self, *args, **kwargs):
        self.serializers_instances = kwargs.pop('serializers', {})
        super(StreamFieldSerializer, self).__init__(*args, **kwargs)

    def get_serializers(self):
        return {}

    def all_serializers(self):
        return {**self.get_serializers(), **self.serializers_instances}

    def to_representation(self, value):
        representation = []
        if value is None:
            return representation
        serializers = self.all_serializers()
        for child in value:
            if child.block.name not in serializers.keys():
                child_representation = child.block.get_api_representation(child.value, context=self.context)
            else:
                ser = serializers[child.block.name](context=self.context)
                child_representation = ser.to_representation(child.value)
            representation.append({
                'type': child.block.name,
                'value': child_representation,
                'id': child.id
            })
        return representation


cached = None


def get_stream_field_serializers():
    global cached
    if cached is not None:
        return cached
    resp_pack = getattr(settings, "REST_PACK", {})
    additional_serializers = resp_pack.get('stream_serializers', {})
    form_serializers = {
        'form_text': 'wagtail_rest_pack.generic_forms.blocks.text_block.InputBlockSerializer',
        'form_group': 'wagtail_rest_pack.generic_forms.blocks.group_block.GroupBlockSerializer',
        'form_submit': 'wagtail_rest_pack.generic_forms.blocks.submit_block.SubmitBlockSerializer',
        'form_open_dialog': 'wagtail_rest_pack.generic_forms.blocks.submit_block.OpenDialogSerializer',
        'form': 'wagtail_rest_pack.generic_forms.view.GetFormBuilderSerializer',
        'richtext': 'wagtail_rest_pack.streamfield.richtext.RichTextSerializer',
        'pagelist': 'wagtail_rest_pack.streamfield.page_list.PageListSerializer',
        'container': 'wagtail_rest_pack.streamfield.container.ContainerSerializer',
        'containers': 'wagtail_rest_pack.streamfield.containers.ContainersSerializer',
        'gallery': 'wagtail_rest_pack.streamfield.gallery.GallerySerializer',
        'embedded': 'wagtail_rest_pack.streamfield.embedded.EmbeddedSerializer',
        'gallery_image': 'wagtail_rest_pack.streamfield.image.GalleryImageSerializer',
        'linkblock': 'wagtail_rest_pack.custom_menu.serializers.LinkBlockSerializer',
        'fblink': 'wagtail_rest_pack.custom_menu.serializers.FacebookLinkSerializer',
        'emaillink': 'wagtail_rest_pack.custom_menu.serializers.EmailLinkSerializer',
        'footerheader': 'wagtail_rest_pack.custom_menu.serializers.HeaderSerializer',
        'contactform': 'wagtail_rest_pack.custom_menu.serializers.ContactFormLinkSerializer',
        'nestedlinkblock': 'wagtail_rest_pack.custom_menu.serializers.LinkBlockSerializer',
        'linkcategory': 'wagtail_rest_pack.custom_menu.serializers.LinkCategorySerializer',
        'categorychildren': 'wagtail_rest_pack.custom_menu.serializers.ChildrenLinksSerializer',
    }
    serializers = {**form_serializers, **additional_serializers}
    classes = {}
    for key, value in serializers.items():
        try:
            module_path, class_name = value.rsplit('.', 1)
            module = import_module(module_path)
            classes[key] = getattr(module, class_name)
        except (ImportError, AttributeError) as e:
            raise ImportError(value)
    cached = classes
    return classes


class SettingsStreamFieldSerializer(StreamFieldSerializer):
    def get_serializers(self):
        return get_stream_field_serializers()



# Generated by Django 4.1.2 on 2022-10-17 17:13

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('comments', '0003_comment_updated_by'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='comment',
            options={'ordering': ['created_on', 'created_by'], 'verbose_name': 'Komentář', 'verbose_name_plural': 'Komentáře'},
        ),
    ]



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class CustomMenuConfig(AppConfig):
    name = 'wagtail_rest_pack.custom_menu'
    verbose_name = _('Custom Page Menu')

    def ready(self):
        pass



from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from wagtail.api.v2.utils import BadRequestError
from django.utils.translation import gettext as _


def create_content_type_id(object_id: str, content_type:str):
    if object_id is None:
        raise BadRequestError(_('`object_id` is not valid'))
    if content_type not in getattr(settings, 'ALLOWED_COMMENTED_CONTENT_TYPES', []):
        raise BadRequestError(_('Given content_type is not allowed to be commented.'))
    if content_type is None or not isinstance(content_type,str):
        raise BadRequestError(_('both content_type and object_id must be supplied.'))
    splitted = content_type.lower().split('.')
    if len(splitted) > 2:
        raise BadRequestError(_('Invalid content_type.'))
    app_label, model = splitted
    content_type_obj = ContentType.objects.get(app_label=app_label,model=model)
    try:
        content_type_obj.get_object_for_this_type(id=int(object_id))
    except content_type_obj.model_class().DoesNotExist:
        raise BadRequestError(_('Given combination of content_type and object_id does not exist'))
    return ContentType.objects.values_list('id').get(app_label=app_label,model=model)[0]



from builtins import super

from wagtail import blocks
from wagtail.embeds.embeds import get_embed
from rest_framework import serializers
from django.utils.translation import gettext as _

def embedded():
    return EmbeddedSerializer.block_definition()

embedded_variants = [
    ('signtranslate', _('Znakovaný překlad')),
    ('video', _('Obyčejné video')),
]
class EmbeddedSerializer(serializers.Serializer):
    block_name = 'embedded'
    definition = serializers.SerializerMethodField('create')

    @staticmethod
    def block_definition():
        return EmbeddedSerializer.block_name, CustomEmbeddedBlock(label=_('Embedded video'))

    def create(self, value):
        isSignTranslate = value['variant'] == 'signtranslate'
        autoplay = isSignTranslate
        useThumbnail = isSignTranslate

        embed = get_embed(value["url"])
        html = embed.html
        min_width = 400
        if embed.width < min_width:
            original_width = embed.width
            original_height = embed.height
            original_ratio = embed.ratio
            embed.width = min_width
            embed.height = int(min_width * original_ratio)
            html = html.replace('width="%s"' % original_width, 'width="300"')
            html = html.replace('height="%s"' % original_height, 'height="%s"' % embed.height)
        if autoplay:
            html = html.replace("feature=oembed", "feature=oembed&autoplay=1")
        return {
            'html': html,
            'thubmnail': embed.thumbnail_url,
            'width': embed.width,
            'height': embed.height,
            'title': embed.title,
            'useThumbnail': useThumbnail
        }

    class Meta:
        fields = ('definition')

class CustomEmbeddedBlock(blocks.StructBlock):
    url = blocks.CharBlock(help_text="Například https://www.youtube.com/watch&v=XqZsoesa55w")
    variant = blocks.ChoiceBlock(embedded_variants, label=_('Sign language translation'), default='signtranslate')

    def __init__(self, *args, **kwargs):
        super(CustomEmbeddedBlock, self).__init__(*args, **kwargs)

    class Meta:
        icon = "media"



# Generated by Django 3.2.3 on 2021-10-03 15:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('custom_menu', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='custommenu',
            name='name',
            field=models.CharField(choices=[('top', 'Horní menu'), ('footer', 'Patička')], max_length=50),
        ),
    ]



from django.conf import settings
from rest_framework.exceptions import ValidationError
from importlib import import_module

def get_client_ip(request):
    x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
    if x_forwarded_for:
        ip = x_forwarded_for.split(',')[0]
    else:
        ip = request.META.get('REMOTE_ADDR')
    return ip

class RecaptchaVerifier:

    def verify(self, request):
        pass


def get_recaptcha_instance() -> RecaptchaVerifier:
    class_str = getattr(settings, 'RECAPTCHA_VERIFIER', None)
    assert class_str is not None, ('Please, specify RECAPTCHA_VERIFIER in settings.')
    try:
        module_path, class_name = class_str.rsplit('.', 1)
        module = import_module(module_path)
        return getattr(module, class_name)()
    except (ImportError, AttributeError) as e:
        raise ImportError(class_str)




from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class StreamfieldConfig(AppConfig):
    name = 'wagtail_rest_pack.relations'
    verbose_name = _('Relations Additions')

    def ready(self):
        from .signal_handlers import register_signal_handlers
        register_signal_handlers()



# Generated by Django 3.2.3 on 2021-10-03 15:55

from django.db import migrations
import wagtail.blocks
import wagtail.fields


class Migration(migrations.Migration):

    dependencies = [
        ('custom_menu', '0003_auto_20211003_1538'),
    ]

    operations = [
        migrations.AddField(
            model_name='custommenu',
            name='content',
            field=wagtail.fields.StreamField([('linkblock', wagtail.blocks.StructBlock([], stream_blocks=[('name', wagtail.blocks.TextBlock(help_text='Name of the link', max_length=50, required=True))]))], default=[]),
        ),
    ]



from django.urls import path

from wagtail_rest_pack.comments.create import CreateCommentAPIView
from wagtail_rest_pack.comments.get import ListCommentAPIView
from wagtail_rest_pack.comments.update import DeleteUpdateCommentAPIView

urlpatterns = [
    path('',  ListCommentAPIView.as_view(), name='list_comments'),
    path('new',  CreateCommentAPIView.as_view(), name='create_comment'),
    path('<int:pk>', DeleteUpdateCommentAPIView.as_view(), name='delete_update_comment'),
]


# Generated by Django 3.1.8 on 2021-05-11 16:25

from django.db import migrations
import wagtail.blocks
import wagtail.fields
import wagtail.images.blocks


class Migration(migrations.Migration):

    dependencies = [
        ('streamfield', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='gallery',
            name='stream',
            field=wagtail.fields.StreamField([('gallery_image', wagtail.blocks.StructBlock([('image', wagtail.images.blocks.ImageChooserBlock())], icon='image', label='Image'))], default=None),
            preserve_default=False,
        ),
    ]



import datetime

from rest_framework import generics, serializers
from rest_framework.serializers import ModelSerializer
from rest_framework.permissions import IsAuthenticated

from wagtail_rest_pack.comments.models import Comment
from wagtail_rest_pack.comments.permission import CommentOwnerOrStaff

from wagtail_rest_pack.exception.handler import custom_exception_handler


class UpdateCommentSerializer(ModelSerializer):
    body = serializers.CharField(max_length=2000, required=True)

    class Meta:
        model = Comment
        fields = ['id', 'body']

    def update(self, instance, validated_data):
        validated_data['updated_on'] = datetime.datetime.now()
        validated_data['updated_by'] = self.context['request'].user
        return super(UpdateCommentSerializer, self).update(instance, validated_data)

class DeleteUpdateCommentAPIView(generics.DestroyAPIView, generics.UpdateAPIView):
    permission_classes = [IsAuthenticated, CommentOwnerOrStaff]
    serializer_class = UpdateCommentSerializer
    lookup_url_kwarg = "pk"

    def get_queryset(self):
        return Comment.objects.all()

    def get_exception_handler(self):
        return custom_exception_handler



# Generated by Django 3.1.8 on 2021-05-10 14:31

from django.db import migrations, models
import wagtail.blocks
import wagtail.fields
import wagtail_rest_pack.generic_forms.blocks.generic_input


class Migration(migrations.Migration):

    dependencies = [
        ('generic_forms', '0009_alter_formbuilder_stream'),
    ]

    operations = [
        migrations.AlterField(
            model_name='formbuilder',
            name='security',
            field=models.CharField(choices=[('recaptcha_or_user', 'Recaptcha or logged user'), ('authenticated_user_only', 'Authenticated user only')], default='recaptcha_or_user', max_length=30),
        ),
        migrations.AlterField(
            model_name='formbuilder',
            name='stream',
            field=wagtail.fields.StreamField([('form_text', wagtail.blocks.StructBlock([('name', wagtail.blocks.TextBlock(help_text='The field name, e.g. email_field', label='Technical name of a field', max_length=30, validators=[wagtail_rest_pack.generic_forms.blocks.generic_input.is_form_field])), ('label', wagtail.blocks.TextBlock(label='Label shown before input', max_length=50)), ('max_length', wagtail.blocks.IntegerBlock(default=100, help_text='Max length of an input', label='Max', max_value=2000, min_value=10)), ('required', wagtail.blocks.ChoiceBlock(choices=[('always', 'Always'), ('anonymous_user_only', 'Not authenticated user only')], label='Condition of field visilibity')), ('placeholder', wagtail.blocks.CharBlock(help_text='Help text shown when nothing filled yet.', max_length=80, required=False)), ('validation', wagtail.blocks.ChoiceBlock(choices=[('none', 'None'), ('email', 'Email')], label='Validation')), ('multiline', wagtail.blocks.BooleanBlock(help_text='Víceřádkový vstup.', required=False))])), ('form_group', wagtail.blocks.StructBlock([('name', wagtail.blocks.CharBlock(max_length=80)), ('row', wagtail.blocks.BooleanBlock(default=False, label='Show on a single line', required=False)), ('required', wagtail.blocks.ChoiceBlock(choices=[('always', 'Always'), ('anonymous_user_only', 'Not authenticated user only')], label='Condition of visibility')), ('stream', wagtail.blocks.StreamBlock([('form_text', wagtail.blocks.StructBlock([('name', wagtail.blocks.TextBlock(help_text='The field name, e.g. email_field', label='Technical name of a field', max_length=30, validators=[wagtail_rest_pack.generic_forms.blocks.generic_input.is_form_field])), ('label', wagtail.blocks.TextBlock(label='Label shown before input', max_length=50)), ('max_length', wagtail.blocks.IntegerBlock(default=100, help_text='Max length of an input', label='Max', max_value=2000, min_value=10)), ('required', wagtail.blocks.ChoiceBlock(choices=[('always', 'Always'), ('anonymous_user_only', 'Not authenticated user only')], label='Condition of field visilibity')), ('placeholder', wagtail.blocks.CharBlock(help_text='Help text shown when nothing filled yet.', max_length=80, required=False)), ('validation', wagtail.blocks.ChoiceBlock(choices=[('none', 'None'), ('email', 'Email')], label='Validation')), ('multiline', wagtail.blocks.BooleanBlock(help_text='Víceřádkový vstup.', required=False))]))]))])), ('form_submit', wagtail.blocks.StructBlock([('name', wagtail.blocks.TextBlock(help_text='Technical name of a field, e.g. email_field', label='Field name', max_length=30, validators=[])), ('text', wagtail.blocks.CharBlock(label='A text button label', max_length=30)), ('action', wagtail.blocks.StreamBlock([('send_email', wagtail.blocks.StructBlock([('sender', wagtail.blocks.CharBlock(label='Technical name of a field containing a sender email address', max_length=50)), ('address', wagtail.blocks.EmailBlock(label='A email address to which a email should be delived to', max_length=150))]))], label='Form action', max_num=1, min_num=1)), ('response', wagtail.blocks.StreamBlock([('form_open_dialog', wagtail.blocks.StructBlock([('title', wagtail.blocks.TextBlock(help_text='title', max_length=40, required=True)), ('text', wagtail.blocks.StreamBlock([('richtext', wagtail.blocks.RichTextBlock(icon='doc-full'))]))])), ('form_open_snack', wagtail.blocks.StructBlock([('text', wagtail.blocks.TextBlock(max_length=50, required=True))]))], label='Form response', max_num=1, min_num=1))]))]),
        ),
    ]



from rest_framework.fields import Field
from wagtail.images.api.fields import ImageRenditionField
from django.conf import settings

class SiteSettingsSerializer(Field):
    """A custom serializer used in Wagtails v2 API."""

    def get_single(self, obj):
        desktop_renderition = getattr(settings, 'IMAGE_LARGE_RENDERITION', 'width-1280')
        mobile_renderition = getattr(settings, 'IMAGE_SMALL_RENDERITION', 'width-480')
        return {
            'large': ImageRenditionField(desktop_renderition).to_representation(obj.image),
            'small': ImageRenditionField(mobile_renderition).to_representation(obj.image),
            'text': obj.text
        }

    def parse_sliders(self, page):
        return [self.get_single(setting.slider) for setting in page.carouselitems.all()]

    def get_sliders_of(self, page):
        if page is None:
            return []
        sliders = self.parse_sliders(page)
        if page.depth == 1:
            return sliders
        if len(sliders) == 0:
            return self.get_sliders_of(page.get_parent().specific)
        return sliders

    def to_representation(self, value):
        sliders = self.get_sliders_of(value.instance)
        return {
            "sliders": sliders
        }


# Generated by Django 3.2.13 on 2022-06-07 15:06

from django.db import migrations
import wagtail.blocks
import wagtail.fields


class Migration(migrations.Migration):

    dependencies = [
        ('custom_menu', '0008_alter_custommenu_stream'),
    ]

    operations = [
        migrations.AlterField(
            model_name='custommenu',
            name='stream',
            field=wagtail.fields.StreamField([('containers', wagtail.blocks.StructBlock([('stream', wagtail.blocks.StreamBlock([('container', wagtail.blocks.StructBlock([('stream', wagtail.blocks.StreamBlock([('nestedlinkblock', wagtail.blocks.StructBlock([('name', wagtail.blocks.TextBlock(help_text='Name of the link', max_length=50, required=True)), ('page', wagtail.blocks.PageChooserBlock(label='A page to be opened')), ('icon', wagtail.blocks.ChoiceBlock(choices=[('none', 'None'), ('pencil', 'Tužka'), ('people', 'Lidi'), ('info', 'Informace'), ('hand', 'Ruka'), ('child', 'Dítě'), ('label', 'Popisek')], label='The icon'))])), ('linkcategory', wagtail.blocks.StructBlock([('name', wagtail.blocks.TextBlock(help_text='Name of the category', max_length=50, required=True)), ('stream', wagtail.blocks.StreamBlock([('nestedlinkblock', wagtail.blocks.StructBlock([('name', wagtail.blocks.TextBlock(help_text='Name of the link', max_length=50, required=True)), ('page', wagtail.blocks.PageChooserBlock(label='A page to be opened')), ('icon', wagtail.blocks.ChoiceBlock(choices=[('none', 'None'), ('pencil', 'Tužka'), ('people', 'Lidi'), ('info', 'Informace'), ('hand', 'Ruka'), ('child', 'Dítě'), ('label', 'Popisek')], label='The icon'))])), ('categorychildren', wagtail.blocks.StructBlock([('parent', wagtail.blocks.PageChooserBlock(label='A page which children should be listed'))]))], label='A nested content of a category')), ('icon', wagtail.blocks.ChoiceBlock(choices=[('none', 'None'), ('pencil', 'Tužka'), ('people', 'Lidi'), ('info', 'Informace'), ('hand', 'Ruka'), ('child', 'Dítě'), ('label', 'Popisek')], label='The icon'))]))], empty=True, label='Content of Column'))], icon='doc-full', label='Sloupec'))], label='Columns'))], icon='container', label='Více sloupců')), ('linkblock', wagtail.blocks.StructBlock([('name', wagtail.blocks.TextBlock(help_text='Name of the link', max_length=50, required=True)), ('page', wagtail.blocks.PageChooserBlock(label='A page to be opened')), ('icon', wagtail.blocks.ChoiceBlock(choices=[('none', 'None'), ('pencil', 'Tužka'), ('people', 'Lidi'), ('info', 'Informace'), ('hand', 'Ruka'), ('child', 'Dítě'), ('label', 'Popisek')], label='The icon'))])), ('linkcategory', wagtail.blocks.StructBlock([('name', wagtail.blocks.TextBlock(help_text='Name of the category', max_length=50, required=True)), ('stream', wagtail.blocks.StreamBlock([('nestedlinkblock', wagtail.blocks.StructBlock([('name', wagtail.blocks.TextBlock(help_text='Name of the link', max_length=50, required=True)), ('page', wagtail.blocks.PageChooserBlock(label='A page to be opened')), ('icon', wagtail.blocks.ChoiceBlock(choices=[('none', 'None'), ('pencil', 'Tužka'), ('people', 'Lidi'), ('info', 'Informace'), ('hand', 'Ruka'), ('child', 'Dítě'), ('label', 'Popisek')], label='The icon'))])), ('categorychildren', wagtail.blocks.StructBlock([('parent', wagtail.blocks.PageChooserBlock(label='A page which children should be listed'))]))], label='A nested content of a category')), ('icon', wagtail.blocks.ChoiceBlock(choices=[('none', 'None'), ('pencil', 'Tužka'), ('people', 'Lidi'), ('info', 'Informace'), ('hand', 'Ruka'), ('child', 'Dítě'), ('label', 'Popisek')], label='The icon'))]))], default=[]),
        ),
    ]




from rest_framework import serializers, generics
from rest_framework.permissions import AllowAny
from wagtail.fields import StreamField
from wagtail_rest_pack.exception.handler import custom_exception_handler
from wagtail_rest_pack.streamfield.serializers import SettingsStreamFieldSerializer
from django.utils.translation import gettext_lazy as _

from .models import FormBuilder
from wagtail.snippets.blocks import SnippetChooserBlock


def form_block():
    return GetFormBuilderSerializer.block_definition()

class GetFormBuilderSerializer(serializers.ModelSerializer):
    block_name = 'form'
    stream = SettingsStreamFieldSerializer()

    @staticmethod
    def block_definition():
        return GetFormBuilderSerializer.block_name, SnippetChooserBlock(target_model='generic_forms.FormBuilder', label=_('form'), icon='form')

    class Meta:
        model = FormBuilder
        fields = ['name', 'display_name', 'description', 'security', 'stream']

